#!/usr/bin/env python3
#
# Parses and checks external VM IPs.

import json
from cmk.agent_based.v2 import Result, Service, State, CheckPlugin, AgentSection


def parse_triton_wedge(string_table):
    lookup = {}

    for row in string_table:
        nic = json.loads(row[0])
        cn_name = nic["cn"]
        vms_in_cn = lookup.setdefault(cn_name, [])
        vms_in_cn.append(nic)

    return lookup


agent_section_triton_wedge = AgentSection(
    name="triton_wedge",
    parse_function=parse_triton_wedge
)


def discover_triton_wedge(section):
    for cn_name, vms in sorted(section.items()):
        yield Service(item=cn_name, parameters={"name": cn_name})


def check_triton_wedge(item, params, section):
    cn_name = params["name"]
    vms = section.get(cn_name)

    if vms is None:
        yield Result(state=State.WARN, summary="Not appearing in NAPI")
        return

    wedged_vms = []

    for vm in vms:
        if vm["wedged"]:
            wedged_vms.append(vm)

    if len(wedged_vms) == 0:
        yield Result(state=State.OK, summary="No wedge detected")
    elif len(wedged_vms) == 1:
        vm = wedged_vms[0]
        summary = "Potential wedge detected for VM %s (%s)" % (vm["vm"], vm["ip"])
        yield Result(state=State.WARN, summary=summary)
    else:
        lst = ", ".join(map(lambda vm: "VM %s (%s)" % (vm["vm"], vm["ip"]), wedged_vms))
        yield Result(state=State.CRIT, summary=f"Likely wedge detected for {lst}") 


check_plugin_triton_wedge = CheckPlugin(
    name="triton_wedge",
    service_name="Triton Wedge CN %s",
    discovery_function=discover_triton_wedge,
    check_function=check_triton_wedge,
    check_default_parameters={},
)
